unit MainUnit;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  StdCtrls, ComCtrls, ExtCtrls;

const
  AM_ENDSTREAM = WM_USER + 99;

type
  TMainForm = class(TForm)
    Panel1: TPanel;
    ReportMemo: TMemo;
    GoButton: TButton;
    StopButton: TButton;
    Label1: TLabel;
    RangeCombo: TComboBox;
    procedure FormCreate(Sender: TObject);
    procedure GoButtonClick(Sender: TObject);
    procedure StopButtonClick(Sender: TObject);
  protected
    procedure AMEndStream(var Msg: TMessage); message AM_ENDSTREAM;
  public
    { Public declarations }
  end;

var
  MainForm: TMainForm;

implementation

{$R *.DFM}

uses
  AIOUSB, DetectUnit;

type
  TUSBAI1616Config = packed record
    ChannelRange: array[$0..$F] of Byte;
    CalibMode: Byte;
    TrigMode: Byte;
    StartStopCh: Byte;
    Oversample: Byte;
    StartStopChEx: Byte; //added for compatibility with the AI-MUX family
  end;

var
  DeviceIndex: LongWord;
  Config: TUSBAI1616Config;
  NextChannel, NumChannels: Integer;
  ChannelVolts: array of Double;
  LogFile: TFileStream;
  LogScanBuffer: AnsiString;
  ScanNum: Integer;

procedure ADCallback(pBuf: PWord; BufSize, Flags, Context: LongWord); cdecl; forward;

function TimestampStr(const ST: TSystemTime): AnsiString;
begin
  with ST do Result := Format('%.2d:%.2d:%.2d on %.2d of %.2d of %.4d', [ wHour, wMinute, wSecond, wDay, wMonth, wYear ]);
end;

procedure TMainForm.FormCreate(Sender: TObject);
var
  DetectForm: TDetectForm;
  Status, PID, ConfigBufSize: LongWord;

begin
  DeviceIndex := diOnly;

  //If you only have the one board, QueryDeviceInfo() with diOnly will succeed
  //with the right PID. If you have more than one board, or haven't plugged it
  //in, it will fail, and this code will pop up the detector form so the user
  //can pick a board or cancel.
  Status := QueryDeviceInfo(DeviceIndex, @PID, nil, nil, nil, nil);
  if (Status <> ERROR_SUCCESS) or not (((PID >= $8040) and (PID <= $8044)) or ((PID >= $8140) and (PID <= $8144))) then begin
    DetectForm := TDetectForm.Create(Self);
    if DetectForm.ShowModal = mrOK then begin
      DeviceIndex := DetectForm.DeviceIndex;
      DetectForm.Free;
    end
    else begin
      DeviceIndex := diNone;
      DetectForm.Free;
      Application.Terminate;
      Exit;
    end;
  end;

  //Set the combo box to the first A/D range.
  RangeCombo.ItemIndex := 0;
  Config.CalibMode := 0; //Take actual data, not internal calibration sources.
  Config.TrigMode := $15; //Scan selected channels each counter rising edge. Leave counter 0 external.
  Config.Oversample := 0; //No oversample.
  ConfigBufSize := SizeOf(Config);
  ADC_SetConfig(DeviceIndex, @Config, ConfigBufSize);
end;

procedure TMainForm.GoButtonClick(Sender: TObject);
var
  Status: LongWord;
  ConfigBufSize: LongWord;
  Hz: Double;
  FN: AnsiString;
  ST: TSystemTime;
  I: Integer;
  RangeCode: Byte;
  StartChannel, EndChannel: Byte;

begin
  ReportMemo.Lines.Add('Acquiring...');

  //Stop the counter, in case it was running.
  Hz := 0;
  CTR_StartOutputFreq(DeviceIndex, 0, @Hz);

  //The ranges in the combo box are listed in order, so that the index is equal to the range code.
  RangeCode := RangeCombo.ItemIndex;
  for I := $0 to $F do
    Config.ChannelRange[I] := RangeCode
  ;

  NumChannels := 64;
  //Select all 64 channels.
  StartChannel := 0;
  EndChannel := NumChannels - 1;
  Config.StartStopCH := (EndChannel shl 4) or (StartChannel and $F);
  Config.StartStopChEx := (EndChannel and $F0) or (StartChannel shr 4);

  ConfigBufSize := SizeOf(Config);
  ADC_SetConfig(DeviceIndex, @Config, ConfigBufSize);

  ADC_SetCal(DeviceIndex, ':AUTO:'); //If the device doesn't support cal, ADC_SetCal just returns an error.

  SetLength(ChannelVolts, 16);
  NextChannel := 0;

  GetLocalTime(ST);
  with ST do FN := Format('%.4d%.2d%.2d_%.2d%.2d%.2d.csv', [ wYear, wMonth, wDay, wHour, wMinute, wSecond ]);
  LogFile := TFileStream.Create(FN, fmCreate);
  LogFile.Free;
  LogFile := TFileStream.Create(FN, fmOpenReadWrite or fmShareDenyWrite);
  LogScanBuffer := '"Scan#"';
  for I := StartChannel to EndChannel do
    LogScanBuffer := LogScanBuffer + (',"Ch ' + IntToStr(I) + '"')
  ;
  ScanNum := 1;
  LogScanBuffer := IntToStr(ScanNum);

  Status := ADC_BulkContinuousCallbackStart(DeviceIndex, 1024, 64, 0, @ADCallback);
  if Status <> ERROR_SUCCESS then begin
    ReportMemo.Lines.Add('Error ' + IntToStr(Status) + ' starting acquisition.');
    LogFile.Free;
    Exit;
  end;

  //Since we've put it in scan mode, this is 1kHz per channel, or 16kHz total.
  Hz := 1000;
  CTR_StartOutputFreq(DeviceIndex, 0, @Hz);

  ReportMemo.Lines.Add('Acquisition started at ' + TimestampStr(ST) + '.');

  RangeCombo.Enabled := False;
  GoButton.Enabled := False;
  StopButton.Enabled := True;
end;

procedure ADCallback(pBuf: PWord; BufSize, Flags, Context: LongWord); cdecl;
var
  Volts: Double;
  iSample: Integer;
  Range: Byte;
  FixedPointCounts: Integer;

begin
  //Note: This is called from an alternate thread context.
  //(Specifically a worker thread spawned by the driver.)

  if BufSize <> 0 then begin
    for iSample := 0 to (BufSize div 2) - 1 do begin
      FixedPointCounts := pBuf^;
      Inc(pBuf);

      Range := Config.ChannelRange[NextChannel];
      if (Range and 1) <> 0 then FixedPointCounts := FixedPointCounts * 2 - $8000;
      if (Range and 2) = 0 then FixedPointCounts := FixedPointCounts * 2;
      if (Range and 4) = 0 then FixedPointCounts := FixedPointCounts * 5;
      Volts := 1/65536 * FixedPointCounts;
      //Because the display can be slow, and this is called from a worker
      //thread, we just load each channel's last reading for display. Any
      //direct processing of the data, like a Fourier transform, would go
      //here instead.
      ChannelVolts[NextChannel] := Volts;

      Inc(NextChannel);
      if NextChannel >= NumChannels then begin
        NextChannel := 0;
        
        LogFile.WriteBuffer(LogScanBuffer[1], Length(LogScanBuffer));
        Inc(ScanNum);
        LogScanBuffer := IntToStr(ScanNum);
      end;
    end;
  end;
  
  if (Flags and 2) <> 0 then begin
    //This tells the main thread we've got the last continuous callback packet.
    PostMessage(MainForm.Handle, AM_ENDSTREAM, 0, 0);
  end;
end;

procedure TMainForm.AMEndStream(var Msg: TMessage);
var
  ST: TSystemTime;

begin
  GetLocalTime(ST);
  ReportMemo.Lines.Add('Stream ended at ' + TimestampStr(ST) + '...');
end;

procedure TMainForm.StopButtonClick(Sender: TObject);
var
  EndStatus, IOStatus: LongWord;

begin
  EndStatus := ADC_BulkContinuousEnd(DeviceIndex, @IOStatus);

  Application.ProcessMessages; //Make sure we got the "stream ended" message first.

  if (EndStatus = ERROR_SUCCESS) and (IOStatus = ERROR_SUCCESS) then
    ReportMemo.Lines.Add('Acquisition complete.')
  else if EndStatus = ERROR_SUCCESS then
    ReportMemo.Lines.Add('Acquisition ended with I/O error ' + IntToStr(IOStatus) + '.')
  else
    ReportMemo.Lines.Add('Error ' + IntToStr(EndStatus) + ' ending acquisition.')
  ;

  GoButton.Enabled := True;
  StopButton.Enabled := False;
  RangeCombo.Enabled := True;
end;

end.
